import datetime
import lotterylib
import es

es.load('examples/auth/basic_auth')
es.load('ba_bank')
ba_bank = es.import_addon('ba_bank')

class ImproperlyConfigured(Exception):
    """
    Exception raised if you use a obviously bad API key or server key.
    """

"""
Load the api-key and server-key for this server/user. I don't do this in load
because I prefer the myapp being in the global scope without having to use the
'global' keyword.
"""
def load_config():
    """
    The layout of our config file is dead simple:
    If the line starts with 'api key:' (case insensitive) the stuff after the
    colon (whitespaces stripped) is considered the api key.
    The same goes for 'server key:' for the server key respectively.
    The default file uses JavaScript style comments so as not to confuse the 
    end user.
    We also do a very simple regex-check of the key if it's possibly correct.
    """
    import path
    import re
    key_pattern = re.compile(r'[a-zA-Z0-9]{64}')
    cfgfile = path.path(__file__).dirname().joinpath('keys.txt')
    lines = cfgfile.lines(retain=False)
    api_key = ''
    server_key = ''
    for line in lines:
        if ':' in line:
            start, end = line.split(':', 1)
            if start.lower().strip() == 'api key':
                api_key = end.strip()
            elif start.lower().strip() == 'server key':
                server_key = end.strip()
    if not key_pattern.match(api_key):
        raise ImproperlyConfigured, "Invalid API Key: '%s'" % api_key
    if not key_pattern.match(server_key):
        raise ImproperlyConfigured, "Invalid Server Key: '%s'" % server_key
    return api_key, server_key
api_key, server_key = load_config()
"""
The application key from esl.ojii.ch. This is constant.
"""
APP_KEY = '8uKO1JyFmFuX5c3w1op60dMYNZNTw1baIlipTr9dBC8DkjYuLcf6786YB2Ly0rEe'

"""
Get an instance of lotterylib.Application for our application, user and server.
"""
myapp = lotterylib.Application(api_key, server_key, APP_KEY)

def create_account(steamid, amount):
    startbalance = int(ba_bank.config['settings']['bank_start_balance']) + amount
    ba_bank.db.execute("INSERT INTO accounts VALUES(?,?,?,0,?)",
                       (steamid, 'unknown', startbalance, datetime.date.today()))

def give(steamid, amount):
    """
    Give 'steamid' amount$ to his ba_bank account. If no account exists, open
    one.
    """
    check = ba_bank.db.query("SELECT * FROM accounts WHERE steamid='%s'" % steamid)
    if not check:
        create_account(steamid, amount)
    else:
        ba_bank.db.execute("UPDATE accounts SET balance=(balance + ?) date=? WHERE steamid=?",
                           (amount, datetime.date.today(), steamid))
    
aliases = {'k': 1000, 'm': 1000000}
def convert(item):
    if item.isdigit():
        return int(item)
    unit = item[-1].lower()
    item = item[:-1]
    if not item.isdigit():
        return 0
    if not unit in aliases:
        return 0
    return aliases[unit] * int(item)

def handle_response(self, response):
    """
    Handle a response from es.ojii.ch.
    If the response is unsuccessful, we just raise the error. Quick'n'dirty.
    If it's successful, we give them what they won
    """
    if not response.success:
        response.raise_error()
    else:
        for item, winners in response.result.iteritems():
            amount = convert(item)
            if amount:
                for winner in self.winners:
                    give(winner, amount)


def es_map_start(ev):
    """
    Whenever a new map starts, we get the new winning queue
    """
    myapp.get_queue(handle_response)
